# Generated by asset_id_files.py

SIDEHOPPERSTATION_MREA = 0x02690DE1

NAME_TO_ID_MREA = {
    "01_SidehopperStation": 0x02690DE1,
}
# Generated by asset_id_files.py

SIDEHOPPERSTATION_MAPA = 0x2BBCF6DC

NAME_TO_ID_MAPA = {
    "01_SidehopperStation": 0x2BBCF6DC,
}

DOCK_NAMES = {
    "01_SidehopperStation": {
    },
}
