# Generated by asset_id_files.py

CROSSFIRECHAOS_MREA = 0x44D34A2E

NAME_TO_ID_MREA = {
    "03_CrossfireChaos": 0x44D34A2E,
}
# Generated by asset_id_files.py

CROSSFIRECHAOS_MAPA = 0x6D06B113

NAME_TO_ID_MAPA = {
    "03_CrossfireChaos": 0x6D06B113,
}

DOCK_NAMES = {
    "03_CrossfireChaos": {
    },
}
