# Generated by asset_id_files.py

FRONTEND04_MREA = 0xCEE3AF0D

NAME_TO_ID_MREA = {
    "FrontEnd04": 0xCEE3AF0D,
}
# Generated by asset_id_files.py

FRONTEND04_MAPA = 0xE7365430

NAME_TO_ID_MAPA = {
    "FrontEnd04": 0xE7365430,
}

DOCK_NAMES = {
    "FrontEnd04": {
    },
}
