# Generated by asset_id_files.py

MULTI_PIPELINE_MREA = 0x845DCF93

NAME_TO_ID_MREA = {
    "04_Multi_Pipeline": 0x845DCF93,
}
# Generated by asset_id_files.py

MULTI_PIPELINE_MAPA = 0xAD8834AE

NAME_TO_ID_MAPA = {
    "04_Multi_Pipeline": 0xAD8834AE,
}

DOCK_NAMES = {
    "04_Multi_Pipeline": {
    },
}
