# Generated by asset_id_files.py

MULTI_SPIRES_MREA = 0x09749B2F

NAME_TO_ID_MREA = {
    "07_Multi_Spires": 0x09749B2F,
}
# Generated by asset_id_files.py

MULTI_SPIRES_MAPA = 0x20A16012

NAME_TO_ID_MAPA = {
    "07_Multi_Spires": 0x20A16012,
}

DOCK_NAMES = {
    "07_Multi_Spires": {
    },
}
