# Generated by asset_id_files.py

AGON_WASTES_MLVL = 0x42B935E4
FRONTEND_MLVL = 0x69802220
GREAT_TEMPLE_MLVL = 0x863FCD72
M01_SIDEHOPPERSTATION_MLVL = 0xA50A80CC
M02_SPIRES_MLVL = 0xAE171602
M03_CROSSFIRECHAOS_MLVL = 0xE3B0C703
M04_PIPELINE_MLVL = 0x233E42BE
M05_SPIDERCOMPLEX_MLVL = 0x406ADD7F
M06_SHOOTINGGALLERY_MLVL = 0x7E19ED26
SANCTUARY_FORTRESS_MLVL = 0x1BAA96C2
TEMPLE_GROUNDS_MLVL = 0x3BFA3EFF
TORVUS_BOG_MLVL = 0x3DFD2249

NAME_TO_ID_MLVL = {
    "Agon Wastes": 0x42B935E4,
    "FrontEnd": 0x69802220,
    "Great Temple": 0x863FCD72,
    "M01_SidehopperStation": 0xA50A80CC,
    "M02_Spires": 0xAE171602,
    "M03_CrossfireChaos": 0xE3B0C703,
    "M04_Pipeline": 0x233E42BE,
    "M05_SpiderComplex": 0x406ADD7F,
    "M06_ShootingGallery": 0x7E19ED26,
    "Sanctuary Fortress": 0x1BAA96C2,
    "Temple Grounds": 0x3BFA3EFF,
    "Torvus Bog": 0x3DFD2249,
}
# Generated by asset_id_files.py

AGON_WASTES_MAPW = 0x38DBF6A5
FRONTEND_MAPW = 0x13E2E161
GREAT_TEMPLE_MAPW = 0xFC5D0E33
M01_SIDEHOPPERSTATION_MAPW = 0xDF68438D
M02_SPIRES_MAPW = 0xD475D543
M03_CROSSFIRECHAOS_MAPW = 0x99D20442
M04_PIPELINE_MAPW = 0x595C81FF
M05_SPIDERCOMPLEX_MAPW = 0x3A081E3E
M06_SHOOTINGGALLERY_MAPW = 0x047B2E67
SANCTUARY_FORTRESS_MAPW = 0x61C85583
TEMPLE_GROUNDS_MAPW = 0x4198FDBE
TORVUS_BOG_MAPW = 0x479FE108

NAME_TO_ID_MAPW = {
    "Agon Wastes": 0x38DBF6A5,
    "FrontEnd": 0x13E2E161,
    "Great Temple": 0xFC5D0E33,
    "M01_SidehopperStation": 0xDF68438D,
    "M02_Spires": 0xD475D543,
    "M03_CrossfireChaos": 0x99D20442,
    "M04_Pipeline": 0x595C81FF,
    "M05_SpiderComplex": 0x3A081E3E,
    "M06_ShootingGallery": 0x047B2E67,
    "Sanctuary Fortress": 0x61C85583,
    "Temple Grounds": 0x4198FDBE,
    "Torvus Bog": 0x479FE108,
}

_DEDICATED_FILES = {
    "Agon Wastes": ".agon_wastes",
    "FrontEnd": ".frontend",
    "Great Temple": ".great_temple",
    "M01_SidehopperStation": ".m01_sidehopperstation",
    "M02_Spires": ".m02_spires",
    "M03_CrossfireChaos": ".m03_crossfirechaos",
    "M04_Pipeline": ".m04_pipeline",
    "M05_SpiderComplex": ".m05_spidercomplex",
    "M06_ShootingGallery": ".m06_shootinggallery",
    "Sanctuary Fortress": ".sanctuary_fortress",
    "Temple Grounds": ".temple_grounds",
    "Torvus Bog": ".torvus_bog",
}


def load_dedicated_file(world_name: str):
    import importlib
    return importlib.import_module(
        _DEDICATED_FILES[world_name],
        ".".join(__name__.split(".")[:-1]),
    )
