# Generated by asset_id_files.py

MULTI_SPIDERCOMPLEX_MREA = 0xE7095052

NAME_TO_ID_MREA = {
    "05_Multi_spiderComplex": 0xE7095052,
}
# Generated by asset_id_files.py

MULTI_SPIDERCOMPLEX_MAPA = 0xCEDCAB6F

NAME_TO_ID_MAPA = {
    "05_Multi_spiderComplex": 0xCEDCAB6F,
}

DOCK_NAMES = {
    "05_Multi_spiderComplex": {
    },
}
