# Generated by asset_id_files.py

SHOOTINGGALLERY_MREA = 0xD97A600B

NAME_TO_ID_MREA = {
    "06_ShootingGallery": 0xD97A600B,
}
# Generated by asset_id_files.py

SHOOTINGGALLERY_MAPA = 0xF0AF9B36

NAME_TO_ID_MAPA = {
    "06_ShootingGallery": 0xF0AF9B36,
}

DOCK_NAMES = {
    "06_ShootingGallery": {
    },
}
